#include "parser.h"
#include <stdarg.h>

//////////////////////////////////////////////////////////////////////////////
// CParser Class
//

CParser::CParser()
{
  m_fpError = NULL;

  m_uiErrorCount = 0;
  m_uiWarningCount = 0;

  m_pCScanner = NULL;
  m_pvAST = NULL;

  m_pyylloc = NULL;
  m_pfyy_Parse = NULL;
}

CParser::~CParser()
{
  Uninit();
}

VOID CParser::Init(CHAR *pcFile, CHAR **ppcDirectories, UINT_32 uiDirectories, 
		   CParser **ppyy_CParser,
		   YYLTYPE *pyylloc, YY_PARSE_TYPE *pfyy_Parse,
		   CScanner **ppyy_CScanner,
		   int *pyyleng,
		   YY_BEGIN_INITIAL_TYPE *pfyy_BeginInitial,
		   YY_GET_CURRENT_BUFFER_TYPE *pfyy_GetCurrentBuffer,
		   YY_CREATE_BUFFER_TYPE *pfyy_CreateBuffer,
		   YY_DELETE_BUFFER_TYPE *pfyy_DeleteBuffer,
		   YY_SWITCH_TO_BUFFER_TYPE *pfyy_SwitchToBuffer,
		   YY_SET_VALUE_INT_TYPE *pfyy_SetValueInt,
		   YY_SET_VALUE_FLOAT_TYPE *pfyy_SetValueFloat,
		   YY_SET_VALUE_STRING_TYPE *pfyy_SetValueString)
{
  ASSERT( pcFile != NULL );
  ASSERT( ppyy_CParser != NULL && pyylloc != NULL && pfyy_Parse != NULL );
  ASSERT( ppyy_CScanner != NULL &&
	  pyyleng != NULL &&
	  pfyy_BeginInitial != NULL &&
	  pfyy_GetCurrentBuffer != NULL &&
	  pfyy_CreateBuffer != NULL &&
	  pfyy_DeleteBuffer != NULL &&
	  pfyy_SwitchToBuffer != NULL &&
	  pfyy_SetValueInt != NULL &&
	  pfyy_SetValueFloat != NULL &&
	  pfyy_SetValueString != NULL );

  m_fpError = stderr;

  m_uiErrorCount = 0;
  m_uiWarningCount = 0;

  m_pCScanner = new CScanner;
  ASSERT( m_pCScanner != NULL );
  m_pCScanner->Init(pcFile, ppcDirectories, uiDirectories, 
		    pyylloc, pyyleng, 
		    pfyy_BeginInitial,
		    pfyy_GetCurrentBuffer,
		    pfyy_CreateBuffer,
		    pfyy_DeleteBuffer,
		    pfyy_SwitchToBuffer,
		    pfyy_SetValueInt,
		    pfyy_SetValueFloat,
		    pfyy_SetValueString);
  *ppyy_CScanner = m_pCScanner;
  
  *ppyy_CParser = this;

  m_pvAST = NULL;

  m_pyylloc = pyylloc;
  m_pfyy_Parse = pfyy_Parse;
}

VOID CParser::Uninit()
{
  if ( m_pCScanner != NULL )
    {
      delete m_pCScanner;
      m_pCScanner = NULL;
    }
}

VOID *CParser::Parse()
{
  // We may not have been able to open the file
  if ( GetErrorCount() == 0 )
    {
      m_pfyy_Parse();
    }

  if ( GetErrorCount() > 0 )
    {
      fprintf(m_fpError, "Parsing Halted: %ld Errors and %ld Warnings\n",
	      GetErrorCount(), GetWarningCount());
      // Clean up the partial AST
      m_pvAST = NULL;
    }
  else
    {
      fprintf(m_fpError, "Parsing Done: %ld Errors and %ld Warnings\n",
	      GetErrorCount(), GetWarningCount());
    }

  return m_pvAST;
}

VOID CParser::HandleError(UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcFormat, ...)
{
  va_list vaArgs;

  m_uiErrorCount++;
  fprintf(m_fpError, "%s: %ld: %ld: Syntax Error: ", 
	  (m_pyylloc->text == NULL) ? ("") : (m_pyylloc->text), 
	  (uiLine == 0) ? (m_pyylloc->first_line) : (uiLine), 
	  (uiColumn == 0) ? (m_pyylloc->first_column) : (uiColumn));

  // Now print the message sent by the program
  va_start(vaArgs, pcFormat);
  vfprintf(m_fpError, pcFormat, vaArgs);
  va_end(vaArgs);

  fprintf(m_fpError, "\n"); 
}

VOID CParser::HandleWarning(UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcFormat, ...)
{
  va_list vaArgs;

  m_uiWarningCount++;
  fprintf(m_fpError, "%s: %ld: %ld: Syntax Warning: ", 
	  (m_pyylloc->text == NULL) ? ("") : (m_pyylloc->text), 
	  (uiLine == 0) ? (m_pyylloc->first_line) : (uiLine), 
	  (uiColumn == 0) ? (m_pyylloc->first_column) : (uiColumn));

  // Now print the message sent by the program
  va_start(vaArgs, pcFormat);
  vfprintf(m_fpError, pcFormat, vaArgs);
  va_end(vaArgs);

  fprintf(m_fpError, "\n"); 
}





