#ifndef __SLIDEPOINT_H__
#define __SLIDEPOINT_H__

#include "slideprimitive.h"

class CSLIDESurface;

//////////////////////////////////////////////////////////////////////
// CSLIDEPoint Class
//

class CSLIDEPoint : public CSLIDEPrimitive
{
//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEPoint();
  virtual ~CSLIDEPoint();

  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcID, 
		    DynPoint *pdynpt);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  VOID InsertIntoBound(CBound *pCBound);

protected:
  VOID CheckDynamic();
  VOID UpdateState();

//////////////////////////////////////////////////
// Rendering
//
public:

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline VOID SetSurfaceID(CHAR *pcSurfaceID);
  inline VOID SetNormal(DynVector *pdynvNormal);
  inline VOID SetTexture(DynPoint *pdynptTexture);

  inline CSLIDESurface *GetSurface();
  inline const Point &GetPoint();
  inline const Vector &GetNormal();
  inline const Point &GetTexture();

  inline BOOL GetBoundDynamic();
  inline VOID SetBoundDynamic(BOOL bBoundDynamic);

//////////////////////////////////////////////////
// Class Variables
//
public:

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcSurfaceID;
  CSLIDESurface *m_pCSLIDESurface;

  DynPoint m_pt;
  DynVector m_vNormal;
  DynPoint m_ptTexture;

  BOOL m_bBoundDynamic;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDEPoint::SetSurfaceID(CHAR *pcSurfaceID)
{
  m_pcSurfaceID = pcSurfaceID;
}

inline VOID CSLIDEPoint::SetNormal(DynVector *pdynvNormal)
{
  m_vNormal.SetAll(pdynvNormal);
}

inline VOID CSLIDEPoint::SetTexture(DynPoint *pdynptTexture)
{
  m_ptTexture.SetAll(pdynptTexture);
}

inline CSLIDESurface *CSLIDEPoint::GetSurface()
{
  return m_pCSLIDESurface;
}

inline const Point &CSLIDEPoint::GetPoint()
{
  return m_pt;
}

inline const Vector &CSLIDEPoint::GetNormal()
{
  return m_vNormal;
}

inline const Point &CSLIDEPoint::GetTexture()
{
  return m_ptTexture;
}

inline BOOL CSLIDEPoint::GetBoundDynamic()
{
  return m_bBoundDynamic;
}

inline VOID CSLIDEPoint::SetBoundDynamic(BOOL bBoundDynamic)
{
  m_bBoundDynamic = bBoundDynamic;
}

#endif // __SLIDEPOINT_H__


