#ifndef __SLIDEWORLD_H__
#define __SLIDEWORLD_H__

#include "slideprimitive.h"
#include "window.h"
#include "timer.h"

class CSLIDERender;
class CSLIDEViewport;
//class CSLIDECamera;
class CSLIDENode;
class CSLIDEInstance;
class CSLIDETransform;
class CSLIDEFace;
class CSLIDEPoint;
class CSLIDESurface;

//////////////////////////////////////////////////////////////////////
// CSLIDEWorld Class
//

class CSLIDEWorld
{
//////////////////////////////////////////////////
// Friends
//
public:

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEWorld();
  virtual ~CSLIDEWorld();
  virtual VOID Init();
  virtual VOID Uninit();

  virtual VOID UninitTcl();

  virtual BOOL Initialized();

protected:
  virtual VOID InitDefaults();

public:  
  virtual BOOL InsertRender(CSLIDERender *pCSLIDERender);
  virtual BOOL InsertWindow(CWindow *pCWindow);
  virtual CWindow *PeekWindow(CHAR *pcID);
  virtual BOOL InsertViewport(CSLIDEViewport *pCSLIDEViewport);
  virtual CSLIDEViewport *PeekViewport(CHAR *pcID);
//  virtual BOOL InsertCamera(CSLIDECamera *pCSLIDECamera);
//  virtual CSLIDECamera *PeekCamera(CHAR *pcID);
  virtual BOOL InsertNode(CSLIDENode *pCSLIDENode);
  virtual CSLIDENode *PeekNode(CHAR *pcID);
  virtual BOOL InsertInstance(CSLIDEInstance *pCSLIDEInstance);
  virtual BOOL InsertTransform(CSLIDETransform *pCSLIDETransform);
  virtual BOOL InsertFace(CSLIDEFace *pCSLIDEFace);
  virtual CSLIDEFace *PeekFace(CHAR *pcID);
  virtual BOOL InsertPoint(CSLIDEPoint *pCSLIDEPoint);
  virtual CSLIDEPoint *PeekPoint(CHAR *pcID);
  virtual BOOL InsertSurface(CSLIDESurface *pCSLIDESurface);
  virtual CSLIDESurface *PeekSurface(CHAR *pcID);
  virtual BOOL InsertGlobalInitTclBlock(CHAR *pcTclBlock);
  virtual BOOL InsertGlobalUpdateTclBlock(CHAR *pcTclBlock);

  virtual BOOL CheckSemantics();

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Iteration Functions
//
public:

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual VOID Preprocess();

protected:
  virtual VOID PreprocessGraphs();
  virtual VOID CreateDynamicsList();

public:
  virtual VOID Update();

protected:
  virtual VOID UpdateTclBlocks();
  virtual VOID UpdateGraphs();

//////////////////////////////////////////////////
// Rendering
//
public:
  virtual VOID RestartDynamics();
  virtual VOID PauseDynamics();
  virtual VOID ScheduleDynamics(INT_32 iSleepTime);
  virtual VOID UnscheduleDynamics();
  virtual VOID HandleDynamics();
  static VOID CBHandleDynamics(ClientData pvClientData);

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  UINT_32 GetErrorCount();
  UINT_32 GetWarningCount();

//////////////////////////////////////////////////
// Class Accessor Functions
//
public:
  static inline Tcl_Interp *GetInterp();
  static inline VOID SetInterp(Tcl_Interp *pInterp);
  static inline CSLIDEWorld *GetWorld();
  static inline VOID SetWorld(CSLIDEWorld *pCSLIDEWorld);
  static inline DOUBLE GetSecondsPerFrameRequested();
  static inline DOUBLE GetUpdateTime();

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static Tcl_Interp *m_pInterp;
  static CSLIDEWorld *m_pCSLIDEWorld;

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CListInt m_listCSLIDERender;

  CListInt m_listCWindow;
  CHashInt m_hashCWindow;
  CListInt m_listCSLIDEViewport;
  CHashInt m_hashCSLIDEViewport;
//  CListInt m_listCSLIDECamera;
//  CHashInt m_hashCSLIDECamera;
  CListInt m_listCSLIDENode;
  CHashInt m_hashCSLIDENode;
  CListInt m_listCSLIDEInstance;
  CListInt m_listCSLIDETransform;
  CListInt m_listCSLIDEFace;
  CHashInt m_hashCSLIDEFace;
  CListInt m_listCSLIDEPoint;
  CHashInt m_hashCSLIDEPoint;
  CListInt m_listCSLIDESurface;
  CHashInt m_hashCSLIDESurface;

  // These will only be evaluated once during Preprocess
  CListInt m_listCSLIDEGlobalInitTclBlock; 
  CListInt m_listCSLIDEGlobalUpdateTclBlock; 

  // We will only call Update on these objects
  CListInt m_listDynamics;

  // Defaults

  // Dynamics
  BOOL m_bDynamicsPaused;
  static DOUBLE m_dSecondsPerFrameRequested;
  static DOUBLE m_dUpdateTime;
  Tk_TimerToken m_ttkWakeUpIDDynamics;
  TimeValue m_tvStart;
  DOUBLE m_dTimeBase;
  UINT_32 m_uiFrame;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline Tcl_Interp *CSLIDEWorld::GetInterp()
{
  return m_pInterp;
}

inline VOID CSLIDEWorld::SetInterp(Tcl_Interp *pInterp)
{
  m_pInterp = pInterp;
}

inline CSLIDEWorld *CSLIDEWorld::GetWorld()
{
  return m_pCSLIDEWorld;
}

inline VOID CSLIDEWorld::SetWorld(CSLIDEWorld *pCSLIDEWorld)
{
  m_pCSLIDEWorld = pCSLIDEWorld;
}

inline DOUBLE CSLIDEWorld::GetSecondsPerFrameRequested()
{
  return m_dSecondsPerFrameRequested;
}

inline DOUBLE CSLIDEWorld::GetUpdateTime()
{
  return m_dUpdateTime;
}

#endif // __SLIDEWORLD_H__


